

import json

def SCAN_CLIQUE (GROUP):
	import click
	@GROUP.group ("SCAN")
	def GROUP ():
		pass
		
	'''
		BRUNCH SCAN BYTES --drive-path /dev/sdb --indexes '[ 0, 99 ]' --bytes-per-plate 512
	'''
	import click
	@GROUP.command ("BYTES")
	@click.option ('--drive-path', required = True, help = '')
	@click.option ('--indexes', required = True, help = '')
	@click.option ('--bytes-per-plate', default = 512, help = '')
	def EXAMPLE (drive_path, indexes, bytes_per_plate):	
		print ("INDEXES", indexes)
		INDEXES = json.loads ('{ "INDEXES": ' + indexes + '}')["INDEXES"]
		
		START_INDEX = INDEXES [0]
		assert (type (START_INDEX) == int)
		
		END_INDEX = INDEXES [1]
		assert (type (END_INDEX) == int)
		
		DRIVE_PATH = drive_path
		
		BYTES_PER_PLATE = bytes_per_plate
		
		#BYTE_STRING = b''
		def PROGRESS (PARAMS):
			#nonlocal BYTE_STRING;
			#BYTE_STRING += PARAMS ['PLATE']
			
			INDEXES = PARAMS ["INDEXES"]
			
			print ()
			print ("INDEXES:", INDEXES[0], "TO", INDEXES[1])
			print (PARAMS ['PLATE'].hex ())
		
		import BRUNCH.SCAN as SCAN
		SCAN.START ({
			"DRIVE PATH": DRIVE_PATH, 
			
			"BYTES INDEXES": [ START_INDEX, END_INDEX ],		
			"BYTES PER PLATE": BYTES_PER_PLATE,
			
			"PROGRESS": PROGRESS
		})	
		
		
		return;

	return;







from .SCULPT 	import SCULPT_OPTIONS
from .SCAN 		import SCAN_CLIQUE

def START ():
	import click
	@click.group ()
	def GROUP ():
		pass


	import click
	@click.command ("example")
	def EXAMPLE ():	
		print ("EXAMPLE")

		return;
	GROUP.add_command (EXAMPLE)

	SCULPT_OPTIONS (GROUP)
	SCAN_CLIQUE (GROUP)
	#BYTES (GROUP)

	GROUP ()







def SCULPT_OPTIONS (GROUP):
	import click
	@GROUP.group ("SCULPT")
	def GROUP ():
		pass
		
	import click
	@GROUP.command ("ENTIRE-DRIVE")
	@click.option ('--drive-path', required = True, help = '')
	@click.option ('--drive-bytes', required = True, help = '')
	def EXAMPLE (port):	
		


		return;
		
		
		
	import click
	@GROUP.command ("BYTES")
	@click.option ('--drive-path', required = True, help = '')
	@click.option ('--index-0', required = True, help = '')
	@click.option ('--index-1', required = True, help = '')
	def EXAMPLE (port):	


		return;

	return;




'''
	VENTURES:
		
		from BRUNCH.ERASE import ERASE
		
		ERASE ({
			"DRIVE PATH": DRIVE_PATH,
			"SCULPT": {
				"ROUTINES": [{
					"BYTES PER PLATE": 512 * 512,
					"BYTES": "ROUTINE_1"
				},{
					"BYTES PER PLATE": 512 * 512,
					"BYTES": "ROUTINE_2"
				}]
			},
			"SCAN": {
				"BYTES PER PLATE": 512 * 512
			}
		})
'''

import BRUNCH.SCAN as SCAN
import BRUNCH.SCULPT as SCULPT

def ERASE (CARGO):



	return




'''
import BRUNCH.SCULPT as SCULPT

SCULPT.START ({
	"DRIVE PATH": DRIVE_PATH,
	
	"BYTES INDEXES": [ 0, 28 ],		
	"BYTES FOR PLATE": b'\xff\xfe\xfd\xfc\xfb\xfa\xf9\xf8\xf7\xf6',
	
	"PROGRESS": PROGRESS
})
'''


def PROGRESS (PARAMS):
	return;


#
#	https://python-jsonschema.readthedocs.io/en/latest/
#
from jsonschema import validate
SCHEMA = {
    "type" : "object",
	"required": [ 
		"DRIVE PATH",
		"BYTES INDEXES", 
		"BYTES FOR PLATE" 
	],
    "properties" : {
		"DRIVE PATH": {
			"type": "string"
		},
		
        "BYTES INDEXES" : { 
			"type" : "array",
		}
    }
}

def START (CARGO):
	validate (
		instance = CARGO, 
		schema = SCHEMA
	)

	DRIVE_PATH = CARGO ["DRIVE PATH"]
	
	BYTES_FOR_PLATE = CARGO ["BYTES FOR PLATE"]
	assert (type (BYTES_FOR_PLATE) == bytes)
	assert (len (BYTES_FOR_PLATE) >= 1)
	
	BYTES_PER_PLATE = len (BYTES_FOR_PLATE)
	BYTES_INDEXES = CARGO ["BYTES INDEXES"]
	
	MEAL_INDEX_START = BYTES_INDEXES [0];
	MEAL_INDEX_END = BYTES_INDEXES [1];

	if ("PROGRESS" in CARGO):
		PROGRESS = CARGO ["PROGRESS"]

	with open (DRIVE_PATH, "wb") as SELECTOR:
		SELECTOR.seek (MEAL_INDEX_START)
		
		print ("OPENED")
		
		#LOOP = 1
		LAST_LOOP = False;
		PLATE = True;
		while PLATE:
			#if (LOOP % 1000 == 0):
			#	print ("SCULPTING")
			#LOOP += 1
		
			PLATE_INDEX_START = SELECTOR.tell ()
			
			#print ("PLATE INDEX START:", PLATE_INDEX_START)
			
			#
			#	CHECK IF A FULL PLATE WOULD 
			#	PUT THE SCANNER PAST THE LAST INDEX
			#
			if (
				(PLATE_INDEX_START + BYTES_PER_PLATE) > MEAL_INDEX_END
			):
				SCULPT_SIZE = MEAL_INDEX_END - PLATE_INDEX_START + 1
				
				LAST_LOOP = True
				
				if (SCULPT_SIZE <= 0):
					print ("SCULPT ENDED, SCULPT SIZE IS 0")
				
					return;
					
				SCULPTURE = BYTES_FOR_PLATE [0:SCULPT_SIZE]
					
			else:
				SCULPT_SIZE = BYTES_PER_PLATE
				SCULPTURE = BYTES_FOR_PLATE

			
			SELECTOR.write (SCULPTURE)
			PLATE_INDEX_END = SELECTOR.tell ()
			
			assert (len (SCULPTURE) == SCULPT_SIZE)
			
			PROGRESS ({
				"SIZE": SCULPT_SIZE,
				"LAST_LOOP": LAST_LOOP,
				"INDEXES": [
					PLATE_INDEX_START,
					PLATE_INDEX_END
				]
			})
			
			if (LAST_LOOP):
				print ("CLOSING SCULPT POINTER") 
			
				SELECTOR.close ()
			
				print ()
				print ("SCULPT IS DONE, LAST BYTE INDEX =", PLATE_INDEX_END)
				print ()
				
				
				
				return;



'''
	import BRUNCH.SCAN.CAN_SCAN as CAN_SCAN
	
	
	[ SCANNED ] = CAN_SCAN.START ({
		"DRIVE PATH": "/dev/sdb",
		
		"BYTE INDEX": 40018597888
	})
'''

def START (CARGO):	
	SCAN_SIZE = 512 * 512;
	BYTE_INDEX = CARGO ["BYTE INDEX"];
	DRIVE_PATH = CARGO ["DRIVE PATH"];

	print ("BYTE INDEX", BYTE_INDEX)
	print ("BYTE INDEX", type (BYTE_INDEX))

	with open (DRIVE_PATH, "rb") as f:
		f.seek (BYTE_INDEX)
		
		print ("TELL", f.tell ())
		PLATE = f.read (SCAN_SIZE)
		
		print ("TELL", f.tell ())
		
		if (len (PLATE) == 0):
			print ("THERE WERE NO BYTES LEFT TO SCAN")
			return [ False, PLATE ]

		return [ True, PLATE ]

	return [ "?", PLATE ]